package com.airfrance.squalecommon.enterpriselayer.facade.config;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.provider.persistence.hibernate.facade.FacadeHelper;
import com.airfrance.jraf.spi.enterpriselayer.IFacade;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.config.AuditFrequencyDAOImpl;
import com.airfrance.squalecommon.daolayer.config.ProjectProfileDAOImpl;
import com.airfrance.squalecommon.daolayer.config.SourceManagementDAOImpl;
import com.airfrance.squalecommon.daolayer.config.StopTimeDAOImpl;
import com.airfrance.squalecommon.datatransfertobject.config.SqualixConfigurationDTO;
import com.airfrance.squalecommon.datatransfertobject.transform.config.AuditFrequencyTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.config.ProjectProfileTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.config.SourceManagementTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.config.StopTimeTransform;
import com.airfrance.squalecommon.datatransfertobject.transform.config.TaskTransform;
import com.airfrance.squalecommon.enterpriselayer.businessobject.config.ProjectProfileBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.config.SourceManagementBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.config.TaskRefBO;

/**
 * Facade pour la configuration Squalix
 */
public class SqualixConfigFacade implements IFacade {

    /**
      * provider de persistence
      */
    private static final IPersistenceProvider PERSISTENTPROVIDER = PersistenceHelper.getPersistenceProvider();

    /**
      * Obtention de la configuration Squalix
      * @return la configuration Squalix
      * @throws JrafEnterpriseException si une erreur survient
      */
    public static SqualixConfigurationDTO getConfig() throws JrafEnterpriseException {
        SqualixConfigurationDTO result = new SqualixConfigurationDTO();
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            //On rcupre les paramtres gnraux:
            Collection stopTimesDTO = getStopTimes(session);
            result.setStopTimes(stopTimesDTO);
            Collection frequenciesDTO = getFrequencies(session);
            result.setFrequencies(frequenciesDTO);
            //On rcupre les profils:
            Collection profilesDTO = getProfiles(session);
            result.setProfiles(profilesDTO);
            //On rcupre les rcuprateurs de sources:
            Collection managersDTO = getSourceManagements(session);
            result.setSourceManagements(managersDTO);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, SqualixConfigFacade.class.getName() + ".getConfig");
        } finally {
            FacadeHelper.closeSession(session, SqualixConfigFacade.class.getName() + ".getConfig");
        }

        return result;
    }

    /**
     * Rcupre les dates limites courantes
     * 
     * @param pSession la session hibernate
     * @return les dates limites sous forme DTO
     * @throws JrafDaoException si erreur
     */
    private static Collection getStopTimes(ISession pSession) throws JrafDaoException {
        StopTimeDAOImpl stopTimeDAO = StopTimeDAOImpl.getInstance();
        Collection stopTimesBO = stopTimeDAO.findAll(pSession);
        Collection stopTimesDTO = StopTimeTransform.bo2dto(stopTimesBO);
        return stopTimesDTO;
    }

    /**
     * Rcupre les frquences d'audit
     * 
     * @param pSession la session hibernate
     * @return lles frquences d'audit sous forme DTO
     * @throws JrafDaoException si erreur
     */
    private static Collection getFrequencies(ISession pSession) throws JrafDaoException {
        AuditFrequencyDAOImpl stopTimeDAO = AuditFrequencyDAOImpl.getInstance();
        Collection frequenciesBO = stopTimeDAO.findAll(pSession);
        Collection frequencieDTO = AuditFrequencyTransform.bo2dto(frequenciesBO);
        return frequencieDTO;
    }

    /**
     * Rcupre les profils courants
     * 
     * @return les profils sous forme DTO
     * @throws JrafEnterpriseException si erreur
     */
    public static Collection getProfiles() throws JrafEnterpriseException {
        ISession session = null;
        Collection result = new ArrayList();
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            result = getProfiles(session);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, SqualixConfigFacade.class.getName() + ".getProfiles");
        } finally {
            FacadeHelper.closeSession(session, SqualixConfigFacade.class.getName() + ".getProfiles");
        }
        return result;
    }

    /**
     * Rcupre les profils courants
     * 
     * @param pSession la session hibernate
     * @return les profils sous forme DTO
     * @throws JrafDaoException si erreur
     */
    private static Collection getProfiles(ISession pSession) throws JrafDaoException {
        ProjectProfileDAOImpl profileDAO = ProjectProfileDAOImpl.getInstance();
        Collection profilesBO = profileDAO.findProfiles(pSession);
        Collection profilesDTO = ProjectProfileTransform.bo2dto(profilesBO);
        return profilesDTO;
    }

    /**
     * Rcupre les rcuprateurs de sources courants
     * 
     * @return les rcuprateurs de sources sous forme DTO
     * @throws JrafEnterpriseException si erreur
     */
    public static Collection getSourceManagements() throws JrafEnterpriseException {
        ISession session = null;
        Collection result = new ArrayList();
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            result = getSourceManagements(session);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, SqualixConfigFacade.class.getName() + ".getSourceManagements");
        } finally {
            FacadeHelper.closeSession(session, SqualixConfigFacade.class.getName() + ".getSourceManagements");
        }
        return result;
    }

    /**
     * Rcupre les rcuprateurs de sources courants
     * 
     * @param pSession la session hibernate
     * @return les rcuprateurs de sources sous forme DTO
     * @throws JrafDaoException si erreur
     */
    private static Collection getSourceManagements(ISession pSession) throws JrafDaoException {
        SourceManagementDAOImpl managerDAO = SourceManagementDAOImpl.getInstance();
        Collection managersBO = managerDAO.findSourceManagemements(pSession);
        Collection managersDTO = SourceManagementTransform.bo2dto(managersBO);
        return managersDTO;
    }

    /**
     * Rcupre la liste des tches configurables
     * 
     * @param pManagerName le nom du rcuprateur de source
     * @param pProfileName le nom du profil
     * @return les tches configurables
     * @throws JrafEnterpriseException si erreur
     */
    public static Collection getConfigurableTasks(String pManagerName, String pProfileName) throws JrafEnterpriseException {
        ISession session = null;
        Collection result = new ArrayList();
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            SourceManagementDAOImpl managerDAO = SourceManagementDAOImpl.getInstance();
            SourceManagementBO managerBO = (SourceManagementBO) managerDAO.findWhereName(session, pManagerName);
            result.addAll(getConfigurableTasks(managerBO.getAnalysisTasks()));
            result.addAll(getConfigurableTasks(managerBO.getTerminationTasks()));
            ProjectProfileDAOImpl profileDAO = ProjectProfileDAOImpl.getInstance();
            ProjectProfileBO profileBO = (ProjectProfileBO) profileDAO.findWhereName(session, pProfileName);
            result.addAll(getConfigurableTasks(profileBO.getAnalysisTasks()));
            result.addAll(getConfigurableTasks(profileBO.getTerminationTasks()));
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, SqualixConfigFacade.class.getName() + ".getConfigurableTasks");
        } finally {
            FacadeHelper.closeSession(session, SqualixConfigFacade.class.getName() + ".getConfigurableTasks");
        }
        return result;
    }

     /**
     * Renvoit les tches de la liste qui sont configurables
     * 
     * @param pTasks une liste de TaskBO
     * @return la collection des tches configurables sous forme DTO
     */
    private static Collection getConfigurableTasks(List pTasks) {
        Collection tasks = new ArrayList();
        TaskRefBO task;
        for (int i = 0; i < pTasks.size(); i++) {
            task = (TaskRefBO) pTasks.get(i);
            if (task.getTask().isConfigurable()) {
                tasks.add(TaskTransform.bo2dto(task));
            }
        }
        return tasks;
    }
}
