package com.airfrance.squalix.tools.cpptest;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.digester.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalecommon.enterpriselayer.businessobject.result.rulechecking.RuleCheckingTransgressionItemBO;
import com.airfrance.squalecommon.util.xml.XmlImport;
import com.airfrance.squalix.core.exception.ConfigurationException;

/**
 * Parseur de rsultat cppTest
 * CppTest gnre ses rsultats sous la forme de fichier XML,
 * cette classe permet de lire le contenu d'un tel fichier.
 * Le fichier de rapport est organis en deux sections : la premire
 * donne la liste des rgles, la seconde la liste des violations rencontres
 */
public class ReportParser extends XmlImport {
    /** Prfixe de fichier */
    private String mFilePrefix;
    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(ReportParser.class);
    /**
     * Constructeur
     */
    public ReportParser() {
        super(LOGGER);
    }

    /**
     * Parsing d'un fichier de rsultat
     * @param pStream stream
     * @param pFilePrefix prfixe de fichier
     * @return collection de rsultats sous la forme
     * d'une map avec en clef le nom de la rgle et en valeur
     * une collection qui donne le dtail de chaque violation sous la forme d'une String
     * @throws ConfigurationException si erreur
     */
    public Map parse(InputStream pStream, String pFilePrefix) throws ConfigurationException {
        Map result = null;
        mFilePrefix = pFilePrefix;
        StringBuffer errors = new StringBuffer();
        Digester digester = preSetupDigester(null, null, errors);
        // Traitement de la liste des rgles
        digester.addCallMethod("Log/Group/Property", "setRule", 1, new Class[] { String.class });
        digester.addCallParam("Log/Group/Property", 0, "name");
        // Traitement du script de compilation
        digester.addCallMethod("Log/Group/Test/Message/Property", "setMessage", 2, new Class[] { String.class, String.class });
        digester.addCallParam("Log/Group/Test/Message/Property", 0, "name");
        digester.addCallParam("Log/Group/Test/Message/Property", 1, "value");
        digester.addCallMethod("Log/Group/Test", "addItem", 1, new Class[] { String.class });
        digester.addCallParam("Log/Group/Test", 0, "status");
        result = new HashMap();
        digester.push(new ResultWrapper(result));
        parse(digester, pStream, errors);
        if (errors.length() != 0) {
            // Erreurs rencontres pendant le parsing
            throw new ConfigurationException(CppTestMessages.getString("error.report.parsing", errors));
        }
        return result;
    }

    /**
     * Interface de rsultat
     *
     */
    class ResultWrapper {
        /** RuleId */
        private String mRuleId;
        /** File */
        private String mFile;
        /** Line */
        private String mLine;
        /** Description */
        private String mDescription;

        /** Rsultat */
        private Map mResult;
        /**
         * Constructeur
         * @param pResult rsultat
         */
        ResultWrapper(Map pResult) {
            mResult = pResult;
        }

        /**
         * 
         * @param pName nom 
         */
        public void setRule(String pName) {
            final String rulePrefix = "Rule:";
            if (pName.startsWith(rulePrefix)) {
                String rule = pName.substring(rulePrefix.length());
                mResult.put(rule, new ArrayList());
            }
        }
        /**
         * Modification des attributs en fonction de <code>pName</code> et formatage de
         * la valeur si besoin
         * @param pName nom 
         * @param pValue valeur
         */
        public void setMessage(String pName, String pValue) {
            if (pName.equals("rule_id")) {
                // l'id de la rgle
                mRuleId = pValue;
            } else if (pName.equals("file")) {
                // le fichier concern
                if (pValue.startsWith(mFilePrefix)) {
                    // On ne prend pas le prfixe du fichier qui n'appartient pas  l'utilisateur
                    pValue = pValue.substring(mFilePrefix.length());
                }
                mFile = pValue;
            } else if (pName.equals("line")) {
                // Affectation du numro de ligne
                mLine = pValue;
            } else if (pName.equals("description")) {
                // description de la transgression
                mDescription = pValue;
            }
        }

        /**
         * Ajout d'un message
         * @param pStatus status de la vrification 
         */
        public void addMessage(String pStatus) {
            // On compose le message  ajouter
            if (pStatus.equals("failure")) {
                ArrayList list = (ArrayList) mResult.get(mRuleId);
                list.add(CppTestMessages.getString("transgression.detail", new Object[] { mFile, mLine, mDescription }));
            }
        }

        /**
         * Ajout d'un item
         * @param pStatus status de la vrification 
         */
        public void addItem(String pStatus) {
            // On cre l'item  ajouter
            if (pStatus.equals("failure")) {
                ArrayList list = (ArrayList) mResult.get(mRuleId);
                RuleCheckingTransgressionItemBO item = new RuleCheckingTransgressionItemBO();
                item.setComponentFile(mFile);
                item.setLine(Integer.parseInt(mLine));
                item.setMessage(CppTestMessages.getString("transgression.detail", new Object[] { RuleCheckingTransgressionItemBO.PATH_KEY, RuleCheckingTransgressionItemBO.LINE_KEY, mDescription }));
                list.add(item);
            }
        }
    }
}
