package com.airfrance.squalix.tools.mccabe;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ClassBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.JspBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.MethodBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAClassMetricsBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAJspMetricsBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAMethodMetricsBO;
import com.airfrance.squalix.util.parser.J2EEParser;
import com.airfrance.squalix.util.parser.LanguageParser;
import com.airfrance.squalix.util.repository.ComponentRepository;

/**
 * Est en charge de remplacer tous les noms des mthodes par les objets 
 * correspondant.<br>
 * Ceci permet d'utiliser les relations proposes par la base de donnes.<br>
 */
public class McCabeAdaptator {

    /**
     * HashMap des sommes des iv(g).
     */
    private HashMap mSumIvg;

    /**
     * Parser
     */
    private LanguageParser mParser;

    /**
     * Repository
     */
    private ComponentRepository mRepository;

    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(McCabeAdaptator.class);

    /**
     * Constructeur
     * @param pParser le parser
     * @param pRepository le repository
     */
    public McCabeAdaptator(LanguageParser pParser, ComponentRepository pRepository) {
        mParser = pParser;
        mRepository = pRepository;
        mSumIvg = new HashMap();
    }

    /**
     * Adapte le bean des rsultats des mthodes et le fait persister.<br>
     * En utilisant nom du composant rcupr depuis le rapport McCabe, on cre
     * la relation avec le composant adquat. 
     * 
     * @param pMethodResult Ensemble de rsultats de la mthode devant tre modifis 
     * et persists.
     * @throws JrafDaoException si erreur
     */
    public void adaptMethodResult(final McCabeQAMethodMetricsBO pMethodResult) throws JrafDaoException {
        LOGGER.debug(McCabeMessages.getString("logs.debug.adapt_method") + pMethodResult.getComponentName());
        // Recherche de la mthode associe
        MethodBO method = mParser.getMethod(pMethodResult.getComponentName(), pMethodResult.getFilename());
        if (null != method) {
            // On renseigne son numro de ligne dans le fichier pour le plugin IDE
            try {
                method.setStartLine(Integer.parseInt(pMethodResult.getStartLine()));
            } catch(NumberFormatException nfe) {
                // Par dfaut on met la ligne  0
                method.setStartLine(0);
            }
            MethodBO persistentMethod = (MethodBO) mRepository.persisteComponent(method);
            String parentKeyName = mRepository.buildKey((ClassBO) persistentMethod.getParent());
            Integer sumIvG = (Integer) mSumIvg.get(parentKeyName);
            if (null == sumIvG) {
                sumIvG = new Integer(pMethodResult.getIvg().intValue());
            } else {
                int value = sumIvG.intValue() + pMethodResult.getIvg().intValue();
                sumIvG = new Integer(value);
            }
            mSumIvg.put(parentKeyName, sumIvG);
            pMethodResult.setComponent(persistentMethod);
        }
    }

    /**
     * Adapte le bean des rsultats des classes et le fait persister.
     * 
     * @param pClassResult Ensemble des rsultats de la classe devant tre modifis et 
     * persists.
     * @throws JrafDaoException si erreur
     */
    public void adaptClassResult(final McCabeQAClassMetricsBO pClassResult) throws JrafDaoException {
        LOGGER.debug(McCabeMessages.getString("logs.debug.adapt_class") + pClassResult.getComponentName());
        ClassBO classBO = mParser.getClass(pClassResult.getComponentName());
        if (null != classBO) {
            ClassBO persistentClassBO = (ClassBO) mRepository.persisteComponent(classBO);
            pClassResult.setComponent(persistentClassBO);
            String keyName = mRepository.buildKey(persistentClassBO);
            Integer sumIvG = (Integer) mSumIvg.get(keyName);
            if (null == sumIvG) {
                sumIvG = new Integer(0);
            }
            pClassResult.setSumivg(sumIvG);
        }
    }

    /**
     * Adapte le bean des rsultats des mthodes au composant JSP et le fait persister.<br>
     * En utilisant le nom du composant rcupr depuis le rapport McCabe, on cre
     * la relation avec le composant adquat. 
     * 
     * @param pJspMethodResult Ensemble de rsultats de la mthode de la JSP
     * @param pDirectoryName le nom du rpertoire racine contenant les pages JSP
     * @param pId l'index du rpertoires des JSPs dans les paramtres du projet
     * 
     * @throws JrafDaoException si erreur
     */
    public void adaptJspResult(McCabeQAJspMetricsBO pJspMethodResult, String pDirectoryName, int pId) throws JrafDaoException {
        LOGGER.debug(McCabeMessages.getString("logs.debug.adapt_jsp_with_method_result") + pJspMethodResult.getComponentName());
        JspBO jsp = ((J2EEParser)mParser).getJsp(pJspMethodResult.getComponentName(), pJspMethodResult.getFileName(), pDirectoryName, pId);
        if(null != jsp) {
            JspBO persistentJsp = (JspBO) mRepository.persisteComponent(jsp);
            pJspMethodResult.setComponent(persistentJsp);
        }
    }
}
