package com.airfrance.squalix.tools.rsm;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ListParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.MapParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ParametersConstants;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.StringParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.ErrorBO;
import com.airfrance.squalix.core.AbstractTask;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.core.exception.ConfigurationException;
import com.airfrance.squalix.util.buildpath.BuildProjectPath;
import com.airfrance.squalix.util.csv.CSVParser;
import com.airfrance.squalix.util.file.FileUtility;
import com.airfrance.squalix.util.parser.LanguageParser;
import com.airfrance.squalix.util.process.ProcessErrorHandler;
import com.airfrance.squalix.util.process.ProcessManager;
import com.airfrance.squalix.util.process.ProcessOutputHandler;

/**
 */
public abstract class RSMTask extends AbstractTask implements ProcessErrorHandler, ProcessOutputHandler, CSVParser.CSVHandler {

    /** Le parser */
    protected CSVParser mParser;

    /**
     * Logger
     */
    protected static final Log LOGGER = LogFactory.getLog(RSMTask.class);

    /**
     * Instance du persisteur RSM
     */
    protected RSMPersistor mPersistor = null;

    /** le parser associ au language */
    protected LanguageParser mLanguageParser;

    /**
     * Configuration de l'outil d'analyse
     */
    protected RSMConfiguration mConfiguration;

    /** Le processManager **/
    protected ProcessManager mProcess;

    /** le fichier contenant les noms des fichiers  analyser */
    protected File mToAnalyseFileList;

    /**
     * Construction des rpertoires  analyser
     * @param pData donnes de la tche
     * @param pProjectParams paramtres du projet
     * @throws ConfigurationException si erreur
     * @return la liste des fichiers  analyser
     */
    private List buildFilesToProcess(TaskData pData, MapParameterBO pProjectParams) throws ConfigurationException {
        // On prend le view path
        String viewPath = (String) pData.getData(TaskData.VIEW_PATH);
        if (viewPath == null) {
            String message = RSMMessages.getString("exception.variable.not_found", TaskData.VIEW_PATH);
            LOGGER.error(message);
            // Renvoi d'une exception de configuration
            throw new ConfigurationException(message);
        }
        // Pour chaque rpertoire source on ajoute celui-ci
        // On rcupre les chemins relatifs des rpertoires contenant les sources du projet
        ListParameterBO sources = getSourcesDirs(pProjectParams);
        if (sources == null) {
            String message = RSMMessages.getString("exception.sources.notset", "");
            LOGGER.error(message);
            // Renvoi d'une exception de configuration
            throw new ConfigurationException(message);
        }
        List sourcesString = new ArrayList(0);
        List sourcesStringBO = sources.getParameters();
        for (int i = 0; i < sourcesStringBO.size(); i++) {
            sourcesString.add(((StringParameterBO) (sourcesStringBO.get(i))).getValue());
        }
        // Prise en compte des rpertoires exclus
        ListParameterBO excludedDirs = getExcludedDirs(pProjectParams);
        if (excludedDirs == null) {
            // On affecte une liste vide
            excludedDirs = new ListParameterBO();
        }
        // Construction des fichiers  traiter
        List extensions = new ArrayList();
        extensions.addAll(Arrays.asList(mConfiguration.getExtensions()));
        extensions.addAll(Arrays.asList(mConfiguration.getHeaders()));
        String[] setOfExtensions = (String[]) (extensions.toArray(new String[extensions.size()]));
        List resultSources = FileUtility.getIncludedFiles(
            viewPath,
            BuildProjectPath.buildProjectPath(viewPath, sources.getParameters()),
            (ListParameterBO) mProject.getParameter(ParametersConstants.INCLUDED_PATTERNS),
            (ListParameterBO) mProject.getParameter(ParametersConstants.EXCLUDED_PATTERNS),
            (ListParameterBO) mProject.getParameter(ParametersConstants.EXCLUDED_DIRS),
            setOfExtensions);
        return resultSources;
    }

    /**
     * Obtention des rpertories exclus
     * @param pProjectParams paramtres du projet
     * @return rpertoires exclus sous la forme de ListParameterBO(StringParameterBO)
     */
    protected ListParameterBO getExcludedDirs(MapParameterBO pProjectParams) {
        return (ListParameterBO) pProjectParams.getParameters().get(ParametersConstants.EXCLUDED_DIRS);
    }
    /**
     * Obtention des sources
     * @param pProjectParams paramtres du projet
     * @return sources sous la forme de ListParameterBO(StringParameterBO)
     */
    protected ListParameterBO getSourcesDirs(MapParameterBO pProjectParams) {
        return (ListParameterBO) pProjectParams.getParameters().get(ParametersConstants.SOURCES);
    }

    /**
     * lance le parsing des classes, la gnration du rapport et le parsing du rapport
     * @throws TaskException en cas d'chec
     */
    public void execute() throws TaskException {
        try {
            // initialise la tache RSM
            initialize();
            // construit la liste des rpertoires  analyser (fichiers sources - exclus) 
            buildDotLstFile(buildFilesToProcess(getData(), mProject.getParameters()));
            LOGGER.info(RSMMessages.getString("logs.analyzing") + mProject.getParent().getName() + " - " + mProject.getName());
            int execResult = parseSource();
            // un nombre ngatif est une erreur d'excution RSM
            if (execResult < 0) {
                throw new TaskException(RSMMessages.getString("rsm.exec.erreur", new Object[] { new Integer(execResult)}));
            }
            // Si le parsing s'est bien droul, on a gnr le rapport
            // On parse maintenant le rapport
            // si le parsing ne s'est pas bien droul, on va tombre dans l'exception
            // On ne gre pas le code retour car le comportement de RSM est trange
            parseReport(mConfiguration.getReportPath());
            // Une fois que tous les rapports ont t gnrs et parss,
            // on peut gnrer les rsultats de niveau projet
            mPersistor.persistProjectResult();
            // positionne les donnes sur la taille du file System
            affectFileSystemSize(mConfiguration.getWorkspace(), true);
            // Lance les oprations de cloture de la tche
            FileUtility.deleteRecursivelyWithoutDeleteRootDirectory(mConfiguration.getWorkspace());
        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * Construit le fichier .txt recensant l'ensemble des fichiers que RSM doit analyser 
     * @param pFilesToAnalyseList la liste des fichiers  analyser
     * @throws IOException en cas d'chec
     */
    private void buildDotLstFile(List pFilesToAnalyseList) throws IOException {
        mToAnalyseFileList = new File(mConfiguration.getInputFile());
        BufferedWriter bw = new BufferedWriter(new FileWriter(mToAnalyseFileList));
        for (Iterator it = pFilesToAnalyseList.iterator(); it.hasNext();) {
            String filePath = (String) it.next();
            bw.write(filePath);
            bw.newLine();
        }
        // ferme le buffer
        bw.close();
    }

    /**
     * @see com.airfrance.squalix.util.process.ProcessErrorHandler#processError(java.lang.String)
     */
    public void processError(String pErrorMessage) {
        LOGGER.error(RSMMessages.getString("logs.error.tools_error") + pErrorMessage);
        ErrorBO error = new ErrorBO();
        error.setInitialMessage(pErrorMessage);
        error.setMessage(RSMMessages.getString("error.processing"));
        error.setLevel(ErrorBO.CRITICITY_FATAL);
        mErrors.add(error);
    }
    
    /**
     *  
     * {@inheritDoc}
     * @param pOutputLine {@inheritDoc}
     * @see com.airfrance.squalix.util.process.ProcessOutputHandler#processOutput(java.lang.String)
     */
    public void processOutput(String pOutputLine) {
        // TODO Auto-generated method stub

    }

    /**
     * Prpare l'environnement d'excution de l'analyse :
     * <ul>
     * <li>Cration du dossier destination des rsultats du parsing</li>
     * <li>Cration du fichier config.pcf</li>
     * </ul>
     * @exception Exception si un probleme d'initialisation apparait
     */
    private void initialize() throws Exception {
        // On rcupre la configuration du module RSM, personnalise
        // avec les paramtres du projet
        mConfiguration = RSMConfiguration.build(mProject, RSMMessages.getString("configuration.file"), getData());
        File workspace = mConfiguration.getWorkspace();
        if (!workspace.exists() || !workspace.isDirectory() || !workspace.canWrite() || !workspace.canRead()) {
            // On va vrifier que le workspace est disponible
            throw new Exception(RSMMessages.getString("exception.no_workspace"));
        }
        mPersistor = new RSMPersistor(mConfiguration, mAudit, getSession(), getData(), mName, mLanguageParser);
        LOGGER.info(RSMMessages.getString("logs.initialized") + mProject.getParent().getName() + " - " + mProject.getName());
    }
    
    /**
     *  
     * {@inheritDoc}
     * @param pLine {@inheritDoc}
     * @see com.airfrance.squalix.util.csv.CSVParser.CSVHandler#processLine(java.util.List)
     */
    public void processLine(List pLine) {
        // TODO Auto-generated method stub

    }

    /**
     * Cre le ProcessManager.
     * On ne fait pas de new mais un set pour implmenter 
     * le pattern IOC pour pouvoir tester sur un environnement windows
     * @param pArguments arguments
     * @param pDirectory rpertoire de lancement
     * @return le ProcessManager normal
     */
    private ProcessManager createProcessManager(String[] pArguments, File pDirectory) {
        return new ProcessManager(pArguments, null, pDirectory);
    }

    /**
     * Parse les fichiers sources afin d'en extraire les mtriques.
     * @return le rsultat de l'excution 
     * @throws Exception si un problme de parsing apparat.
     */
    private int parseSource() throws Exception {
        // le rsultat renvoy
        int resultParse = 0;
        // Execute le parsing des sources avec RSM
        String[] parseCommand = new String[mConfiguration.getParseParameters().length + 1];
        parseCommand[0] = mConfiguration.getExecCommand();
        // Parse tous les rpertoires sources sauf les rpertoires exclus un par un
        for (int i = 1; i < parseCommand.length; i++) {
            String param = mConfiguration.getParseParameters()[i - 1];
            // On construit la commande
            // On met en forme la liste des paramtres pour la passer au process
            if ("-O".equals(param)) {
                param += mConfiguration.getReportPath();
            } else {
                if ("-F".equals(param)) {
                    param += mConfiguration.getInputFile();
                }
            }
            parseCommand[i] = param;
        }
        LOGGER.info(RSMMessages.getString("logs.running_parsing.command", parseCommand));
        LOGGER.info(RSMMessages.getString("logs.running_parsing", new Object[] { mProject.getParent().getName(), mProject.getName()}));
        mProcess = createProcessManager(parseCommand, mConfiguration.getWorkspace());
        // On veut grer les informations lances par le processus en sortie
        mProcess.setOutputHandler(this);
        // On cherche  avoir un comportement synchrone pour tre sr de ne pas
        // avoir un tat des donnes incohrent
        resultParse = mProcess.startProcess(this);

        LOGGER.info(RSMMessages.getString("logs.return_parsing", new Object[] { mProject.getParent().getName(), mProject.getName(), new Integer(resultParse)}));
        return resultParse;
    }

    /**
    
    /**
     * Analyse un rapport de mtriques RSM
     * @param pReport rapport  parser
     * @throws Exception si erreur
     */
    private void parseReport(final String pReport) throws Exception {
        mPersistor.parseReport(pReport, getData());
    }

    /**
     * @return la configuration
     */
    public RSMConfiguration getConfiguration() {
        return mConfiguration;
    }

    /**
     * @param pConfiguration la configuration
     */
    public void setConfiguration(RSMConfiguration pConfiguration) {
        mConfiguration = pConfiguration;
    }

}