package com.airfrance.squalix.util.parser;

import java.util.TreeSet;

import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AbstractComplexComponentBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ClassBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.MethodBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.PackageBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;

/**
 * Parse les noms de type C++ et les remplace par les objets correspondants.
 */
public class CppParser implements LanguageParser {
    /** Nom du package par dfaut */
    private static final String DEFAULT_PACKAGE_NAME = "{C}";
    /** Nom de la classe par dfaut */
    private static final String DEFAULT_CLASS_NAME = "{C}";
    /**
     * Pattern pour reprer une zone d'initialisation statique
     */
    private static final String STATIC_INITIALIZER_PATTERN = "^.*#StaticInitializer#[0-9]*$";

    /** Projet sur lequel est ralis l'analyse */
    private ProjectBO mProject;

    /** Ensemble des classes connues
     *  Cel permet de renseigner les noms de classe connues pour permettre
     *  une distinction entre un namespace et un nom de classe. McCabe utilise
     *  l'oprateur de qualification :: aussi bien pour une classe imbrique que
     *  pour un namespace
     */
    private TreeSet mKnownClasses = new TreeSet();

    /**
     * Constructeur
     * 
     * @param pProject le projet sur lequel est ralis l'analyse
     */
    public CppParser(ProjectBO pProject) {
        mProject = pProject;
    }

    /**
     * Ajout d'une classe connue
     * @param pClassName nom de la classe
     */
    public void addKnownClass(String pClassName) {
        StringBuffer context = new StringBuffer();
        String className = geRelativeElementName(pClassName, context);
        mKnownClasses.add(className);
    }

    /*################ Dcomposition et transformation en objet correspondant ################*/

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.util.parser.LanguageParser#getMethod(java.lang.String, java.lang.String)
     */
    public MethodBO getMethod(String pAbsoluteMethodName, String pFileName) {
        // On rcupre le nom de la mthode
        MethodBO methodBO;
        try {
            String methodName = getMethodName(pAbsoluteMethodName);
            methodBO = new MethodBO(methodName);
            String nameWithoutParameters;
            // cas des mthodes 'main' C++ inscrite sans signature par McCabe
            if (methodName.equals("main") || methodName.matches("main_#.*")) {
                nameWithoutParameters = methodName;
            } else {
                nameWithoutParameters = pAbsoluteMethodName.substring(0, pAbsoluteMethodName.lastIndexOf("("));
            }
            int classIndex = nameWithoutParameters.lastIndexOf("::");
            String absoluteClassName;
            // On rattache les fonctions  une classe fictive
            if (classIndex == -1) {
                absoluteClassName = DEFAULT_CLASS_NAME;
            } else {
                absoluteClassName = nameWithoutParameters.substring(0, nameWithoutParameters.lastIndexOf("::"));
            }
            ClassBO parent = getClass(absoluteClassName);
            methodBO.setParent(parent);
            methodBO.setLongFileName(pFileName);
        } catch (StringIndexOutOfBoundsException e) {
            // cas d'un nom de mthode mal form - bug McCabe sur l'oprateur C++ () 
            methodBO = null;
        }
        return methodBO;
    }

    /** 
     * {@inheritDoc}
     * @see com.airfrance.squalix.util.parser.LanguageParser#getClass(java.lang.String)
     */
    public ClassBO getClass(String pAbsoluteClassName) {
        // On rcupre le nom de la classe
        StringBuffer context = new StringBuffer();
        String className = geRelativeElementName(pAbsoluteClassName, context);
        ClassBO classBO = new ClassBO(className);
        AbstractComplexComponentBO current = classBO;
        // Le contexte est soit un nom de classe, soit un nom de package
        // On le dcompose entirement
        while (context.length() > 0) {
            AbstractComplexComponentBO parent = null;
            String parentName = geRelativeElementName(context.toString(), context);
            // On doit maintenant deviner si le parentName est une classe ou un namespace
            if (isClassName(parentName)) {
                parent = new ClassBO(parentName);
            } else {
                parent = new PackageBO(parentName);
            }
            current.setParent(parent);
            current = parent;
        }
        AbstractComplexComponentBO parent = null;
        // On rcupre le parent associ
        if (current instanceof ClassBO) {
            parent = getPackage(DEFAULT_PACKAGE_NAME);
        } else {
            parent = mProject;
        }
        current.setParent(parent);
        return classBO;
    }

    /**
     * @param parentName nom de classe
     * @return true si le nom est considr comme tant une classe
     */
    private boolean isClassName(String parentName) {
        // Un nom est supps tre une classe s'il s'agit d'un template
        // ou s'il s'agit d'un nom de classe dj enregistr
        return parentName.indexOf('>') > 0 || mKnownClasses.contains(parentName);
    }

    /**
     * Cre le package dsign par les paramtres.
     * 
     * @param pPackageName le nom absolu du package
     * @return le PackageBO dsign par les paramtres.
     */
    private PackageBO getPackage(String pPackageName) {
        PackageBO packageBO = new PackageBO(getPackageName(pPackageName));
        String parentName = getParentName(pPackageName);
        if (null == parentName) {
            packageBO.setParent(mProject);
        } else {
            PackageBO parent = getPackage(parentName);
            packageBO.setParent(parent);
        }
        return packageBO;
    }

    /**
     * Retourne la chane pAbsoluteName avant le dernier "::" ou null si il n'y a pas de "::"
     * 
     * @param pAbsoluteName le nom absolu du fils
     * @return le nom absolu du parent
     */
    public String getParentName(String pAbsoluteName) {
        String parent = null;
        int lastDot = pAbsoluteName.lastIndexOf("::");
        if (lastDot != -1) {
            parent = pAbsoluteName.substring(0, lastDot);
        }
        return parent;
    }

    /**
     * Retourne le nom du package le plus  droite
     * 
     * @param pPackageName le nom absolu du package
     * @return le nom du dernier package
     */
    private String getPackageName(String pPackageName) {
        String[] splittingPackageName = pPackageName.split("\\.");
        return splittingPackageName[splittingPackageName.length - 1];
    }

    /**
     * Dcompose le nom entirement qualifi d'une mthode afin de rcuprer
     * son nom relatif.
     * @param pAbsoluteMethodName le nom entirement qualifi d'une mthode
     * @return le nom relatif de la mthode
     */
    protected String getMethodName(String pAbsoluteMethodName) {
        String relativeMethodName = "";
        if (pAbsoluteMethodName.matches(STATIC_INITIALIZER_PATTERN)) {
            // S'il s'agit d'une zone d'initialisation statique,
            // on la considre comme une mthode
            relativeMethodName = getStaticInitializerName(pAbsoluteMethodName);
        } else if (pAbsoluteMethodName.equals("main") || pAbsoluteMethodName.matches("main_#.*")) {
            // cas des mthodes 'main' C++ inscrite sans signature par McCabe
            relativeMethodName = pAbsoluteMethodName;
        } else {
            StringBuffer context = new StringBuffer();
            relativeMethodName = geRelativeElementName(pAbsoluteMethodName.substring(0, pAbsoluteMethodName.lastIndexOf("(")), context);
            relativeMethodName += pAbsoluteMethodName.substring(pAbsoluteMethodName.lastIndexOf("("));
        }
        return relativeMethodName;
    }

    /**
     * Dcompose le nom entirement qualifi d'une classe afin de rcuprer son
     * nom relatif.
     * @param pAbsoluteName le nom entirement qualifi de la mthode
     * @param pContext contexte de dfinition de la classe (soit une classe englobante
     * soit un namespace)
     * (ou le nom entirement qualifi de la mthode sans les paramtres)
     * @return le nom relatif de la classe (ou de la mthode sans ses paramtres)
     */
    protected String geRelativeElementName(String pAbsoluteName, StringBuffer pContext) {
        // normal pattern = .+ car il faut aussi prendre en compte les destructeurs
        // et la redfinition des oprateurs
        String normalPattern = ".+$";
        String specialPattern = ".*<.*>.*$";
        String name = "";
        pContext.setLength(0);
        // On commence par tester sur le pattern des templates
        // car le pattern de class simple englobe tout
        // Cas d'une classe base sur des templates
        if (pAbsoluteName.matches(specialPattern)) {
            // Cas complexe utilisant des templates
            int dbpIndex = pAbsoluteName.lastIndexOf("::");
            int gtIndex = pAbsoluteName.lastIndexOf(">");
            if (dbpIndex > gtIndex) {
                // Si le dernier :: est aprs le dernier >
                // On rcupre la chane aprs les ::
                name = pAbsoluteName.substring(dbpIndex + 2);
                pContext.append(pAbsoluteName.substring(0, dbpIndex));
            } else {
                // Sinon traitement spcial
                name = getSpecialCppName(pAbsoluteName, pContext);
            }
        } else {
            // Cas simple d'une classe qui n'utilise pas de template
            if (pAbsoluteName.matches(normalPattern)) {
                int index = pAbsoluteName.lastIndexOf(":");
                name = pAbsoluteName.substring(index + 1);
                if (index > 0) {
                    pContext.append(pAbsoluteName.substring(0, index - 1));
                }
            } else { // Ne devrait jamais arriver
                name = DEFAULT_CLASS_NAME;
            }
        }
        // Enfin, on nettoie le nom complet
        return clearName(name);
    }

    /**
     * Traite les nom spciaux, c'est--dire contenant des dclarations
     * ou utilisation de templates.
     * @param pAbsoluteName le nom  traiter.
     * @param pContext contexte de dfinition de la classe (soit une classe englobante
     * soit un namespace)
     * @return le nom simplement qualifi.
     */
    private String getSpecialCppName(String pAbsoluteName, StringBuffer pContext) {
        int count = 0;
        String className = pAbsoluteName;
        for (int i = 0; i < pAbsoluteName.length(); i++) {
            // On analyse chacun des caractres
            if (count == 0 && pAbsoluteName.charAt(i) == ':' && pAbsoluteName.charAt(++i) == ':') {
                // Si on trouve :: en dehors d'un template, on rcupre ce qu'il y a aprs les ::
                className = pAbsoluteName.substring(i + 1);
                pContext.append(pAbsoluteName.substring(0, i - 1));
            } else if (pAbsoluteName.charAt(i) == '<') {
                // On entre dans un template, donc on attend la fin du template
                count++;
            } else if (pAbsoluteName.charAt(i) == '>') {
                count--;
            }
        }
        return className;
    }

    /**
     * Rcupre le nom de la mthode s'il s'agit d'une zone d'initialisation.
     * 
     * @param pAbsoluteName le nom  traiter.
     * @return le nom simplement qualifi.
     * 
     */
    private String getStaticInitializerName(String pAbsoluteName) {
        int index = pAbsoluteName.lastIndexOf(".");
        String result = pAbsoluteName.substring(++index);
        return result;
    }

    /**
     * Enlve les caractres spciaux  la fin d'un nom de classe, package, mthode.<br>
     * Les caractres spciaux sont les sparateurs Java et C++ : 
     * <ul>
     * <li>.</li>
     * <li>:</li>
     * <li>$</li>
     * </ul>
     * Ainsi que les caractres de chemins : 
     * <ul>
     * <li>/</li>
     * <li>\</li>
     * </ul>
     * 
     * @param pName le nom  nettoyer
     * @return le nom sans caractre spcial en fin.
     */
    public static String clearName(String pName) {
        String badEnd = ".*[/.:\\$#]$";
        while (pName.matches(badEnd)) {
            pName = pName.substring(0, pName.length() - 1);
        }
        return pName;
    }
}
