package com.airfrance.squaleide.preferences;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;

import com.airfrance.squaleide.util.PluginUtils;

/**
 * Champ permettant d'ajouter ou de supprimer des champs de type String afin de crer
 * une liste de String dans les prfrences.
 */
public class AddRemoveListFieldEditor extends FieldEditor {
    /** Bouton pour ajouter un tag */
    private static final String DEFAULT_ADD_LABEL = "Add";
    /** bouton pour supprimer des tags */
    private static final String DEFAULT_REMOVE_LABEL = "Remove selection";
    /** Sparateur par dfaut des champs */
    private static final String DEFAULT_SEPERATOR = ";";

    /** La taille de la liste par dfaut */
    private static final int LIST_HEIGHT_IN_DLUS = 80;

    /** Conteneur des champs d'dition. */
    private Composite top;
    /** Liste des tags */
    private List list;
    /** Le text field pour ajouter un nouveau tag. */
    private Text textField;
    /** Bouton pour ajouter un tag */
    private Button add;
    /** Bouton pour supprimer un slection de tags */
    private Button remove;
    /** Chane utilise pour sparer les tags dans la chane reprsentant la liste */
    private String separator = DEFAULT_SEPERATOR;

    /**
     * Constructeur par dfaut
     * @param name le nom du champs
     * @param labelText le texte pour le label
     * @param parent le conteneur parent
     */
    public AddRemoveListFieldEditor(String name, String labelText, Composite parent) {
        super(name, labelText, parent);
    }

    /**
     * Constructeur spcifique
     * @param name le nom du champs
     * @param labelText le texte pour le label
     * @param addButtonText le label du bouton d'ajout
     * @param removeButtonText le label du bouton de suppression
     * @param parent le conteneur parent
     */
    public AddRemoveListFieldEditor(String name, String labelText, String addButtonText, String removeButtonText, Composite parent) {
        super(name, labelText, parent);
        setAddButtonText(addButtonText);
        setRemoveButtonText(removeButtonText);
    }

    /** 
     * {@inheritDoc}
     * @param numColumns {@inheritDoc}
     * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
     */
    protected void adjustForNumColumns(int numColumns) {
        ((GridData) top.getLayoutData()).horizontalSpan = numColumns;
    }

    /** 
     * {@inheritDoc}
     * @param parent {@inheritDoc}
     * @param numColumns {@inheritDoc}
     * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid(org.eclipse.swt.widgets.Composite, int)
     */
    protected void doFillIntoGrid(Composite parent, int numColumns) {
        top = parent;

        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = numColumns;
        top.setLayoutData(gd);

        Label label = getLabelControl(top);
        GridData labelData = new GridData();
        labelData.horizontalSpan = numColumns;
        label.setLayoutData(labelData);

        list = new List(top, SWT.BORDER);

        // On cre la grille qui va nous permettre de placer nos lments
        GridData listData = new GridData(GridData.FILL_HORIZONTAL);
        listData.heightHint = convertVerticalDLUsToPixels(list, LIST_HEIGHT_IN_DLUS);
        listData.horizontalSpan = numColumns;

        list.setLayoutData(listData);
        list.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                selectionChanged();
            }
        });

        // On cre un conteneur pour les boutons et le textField
        Composite addRemoveGroup = new Composite(top, SWT.NONE);

        GridData addRemoveData = new GridData(GridData.FILL_HORIZONTAL);
        addRemoveData.horizontalSpan = numColumns;
        addRemoveGroup.setLayoutData(addRemoveData);

        GridLayout addRemoveLayout = new GridLayout();
        addRemoveLayout.numColumns = numColumns;
        addRemoveLayout.marginHeight = 0;
        addRemoveLayout.marginWidth = 0;
        addRemoveGroup.setLayout(addRemoveLayout);

        // Le conteneur pour les deux boutons
        Composite buttonGroup = new Composite(addRemoveGroup, SWT.NONE);
        buttonGroup.setLayoutData(new GridData());

        GridLayout buttonLayout = new GridLayout();
        buttonLayout.marginHeight = 0;
        buttonLayout.marginWidth = 0;
        buttonGroup.setLayout(buttonLayout);

        // Le bouton d'ajout.
        add = new Button(buttonGroup, SWT.NONE);
        add.setText(DEFAULT_ADD_LABEL);
        add.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                add();
            }
        });
        GridData addData = new GridData(GridData.FILL_HORIZONTAL);
        addData.heightHint = convertVerticalDLUsToPixels(add, IDialogConstants.BUTTON_HEIGHT);
        addData.widthHint = convertHorizontalDLUsToPixels(add, IDialogConstants.BUTTON_WIDTH);
        add.setLayoutData(addData);

        // Le bouton de suppression
        remove = new Button(buttonGroup, SWT.NONE);
        remove.setEnabled(false);
        remove.setText(DEFAULT_REMOVE_LABEL);
        remove.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                list.remove(list.getSelectionIndex());
                selectionChanged();
            }});
        GridData removeData = new GridData(GridData.FILL_HORIZONTAL);
        removeData.heightHint = convertVerticalDLUsToPixels(remove, IDialogConstants.BUTTON_HEIGHT);
        removeData.widthHint = convertHorizontalDLUsToPixels(remove, IDialogConstants.BUTTON_WIDTH);
        remove.setLayoutData(removeData);

        // Le textField
        textField = new Text(addRemoveGroup, SWT.BORDER);

        GridData textData = new GridData(GridData.FILL_HORIZONTAL);
        textData.horizontalSpan = numColumns - 1;
        textData.verticalAlignment = GridData.BEGINNING;
        textField.setLayoutData(textData);
    }

    /** 
     * Charge la liste
     * @see org.eclipse.jface.preference.FieldEditor#doLoad()
     */
    protected void doLoad() {
        String items = getPreferenceStore().getString(getPreferenceName());
        setList(items);
    }

    /** 
     * Charge la liste par dfaut
     * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
     */
    protected void doLoadDefault() {
        String items = getPreferenceStore().getDefaultString(getPreferenceName());
        setList(items);
    }

    /**
     * Parse la chane afin de sparer les tags d'aprs le sparateur dfinit
     * afin de crer le tableau des tags
     * @param items les tags
     */
    private void setList(String items) {
        String[] itemArray = PluginUtils.parseString(items, separator);
        list.setItems(itemArray);
    }

    /** 
     * Sauvegarde la liste des tags sous forme de chane dans le preferenceStore
     * @see org.eclipse.jface.preference.FieldEditor#doStore()
     */
    protected void doStore() {
        String s = PluginUtils.createListString(list.getItems(), separator);
        if (s != null) {
            getPreferenceStore().setValue(getPreferenceName(), s);
        }
    }

    /** 
     * {@inheritDoc}
     * @return {@inheritDoc}
     * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
     */
    public int getNumberOfControls() {
        // Le bouton et le textField.
        return 2;
    }

    /**
     * Ajoute la chane du textField dans la liste des tags
     */
    private void add() {
        String tag = textField.getText();
        // On ajoute si celui-ci est renseign
        if (tag != null && tag.length() > 0) {
            list.add(tag);
        }
        // On reset
        textField.setText("");
    }

    /**
     * Modifie le label du bouton d'ajout
     * @param text le label
     */
    public void setAddButtonText(String text) {
        add.setText(text);
    }

    /**
     * Modifie le label du bouton de suppression
     * @param text le label
     */
    public void setRemoveButtonText(String text) {
        remove.setText(text);
    }

    /**
     * Modifie le sparateur des tags
     * @param newSeperator le sparateur
     */
    public void setSeparator(String newSeperator) {
        this.separator = newSeperator;
    }

    /**
     * Permet de rendre actif ou inactif le bouton de suppression
     */
    private void selectionChanged() {
        int index = list.getSelectionIndex();
        remove.setEnabled(index >= 0);
    }
}
